import React from 'react';
import { View, ActivityIndicator, Text, ScrollView } from 'react-native';
import { getHighlighter } from './SyntaxContext.js';

var __defProp = Object.defineProperty;
var __defProps = Object.defineProperties;
var __getOwnPropDescs = Object.getOwnPropertyDescriptors;
var __getOwnPropSymbols = Object.getOwnPropertySymbols;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __propIsEnum = Object.prototype.propertyIsEnumerable;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __spreadValues = (a, b) => {
  for (var prop in b || (b = {}))
    if (__hasOwnProp.call(b, prop))
      __defNormalProp(a, prop, b[prop]);
  if (__getOwnPropSymbols)
    for (var prop of __getOwnPropSymbols(b)) {
      if (__propIsEnum.call(b, prop))
        __defNormalProp(a, prop, b[prop]);
    }
  return a;
};
var __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));
var __async = (__this, __arguments, generator) => {
  return new Promise((resolve, reject) => {
    var fulfilled = (value) => {
      try {
        step(generator.next(value));
      } catch (e) {
        reject(e);
      }
    };
    var rejected = (value) => {
      try {
        step(generator.throw(value));
      } catch (e) {
        reject(e);
      }
    };
    var step = (x) => x.done ? resolve(x.value) : Promise.resolve(x.value).then(fulfilled, rejected);
    step((generator = generator.apply(__this, __arguments)).next());
  });
};
function log(message, ...args) {
  {
    console.log(`[SyntaxHighlighter] ${message}`, ...args);
  }
}
function splitTokens(token) {
  const patterns = [
    // JSX/TSX patterns
    { regex: /(<\/?\w+)|(\{[^}]*\})|(\/>)|(>)/g, color: "#85e89d" },
    // JSX tags and expressions
    { regex: /\b(className|onMouseEnter|onMouseLeave)\b/g, color: "#79b8ff" },
    // JSX attributes
    { regex: /['"`].*?['"`]/g, color: "#9ecbff" },
    // Strings
    { regex: /\b(return|const|let|var|function)\b/g, color: "#f97583" },
    // Keywords
    { regex: /\b(true|false|null|undefined)\b/g, color: "#79b8ff" },
    // Constants
    { regex: /[(){}[\]]/g, color: "#e1e4e8" }
    // Brackets
  ];
  const result = [];
  let lastIndex = 0;
  const content = token.content;
  const matches = [];
  patterns.forEach(({ regex, color }) => {
    regex.lastIndex = 0;
    let match;
    while ((match = regex.exec(content)) !== null) {
      matches.push({ pattern: regex, color, match });
    }
  });
  matches.sort((a, b) => a.match.index - b.match.index);
  matches.forEach((matchInfo) => {
    const { match, color } = matchInfo;
    const startPos = match.index;
    if (startPos > lastIndex) {
      result.push({
        content: content.slice(lastIndex, startPos),
        color: token.color,
        fontStyle: token.fontStyle,
        offset: token.offset + lastIndex
      });
    }
    result.push({
      content: match[0],
      color,
      fontStyle: token.fontStyle,
      offset: token.offset + startPos
    });
    lastIndex = startPos + match[0].length;
  });
  if (lastIndex < content.length) {
    result.push({
      content: content.slice(lastIndex),
      color: token.color,
      fontStyle: token.fontStyle,
      offset: token.offset + lastIndex
    });
  }
  return result.filter((t) => t.content.length > 0);
}
const defaultHighlightLines = [];
const SyntaxHighlighter = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines
}) => {
  var _a;
  const [loading, setLoading] = React.useState(true);
  const [error, setError] = React.useState(null);
  const [tokens, setTokens] = React.useState([]);
  React.useEffect(() => {
    const highlight = () => __async(void 0, null, function* () {
      try {
        log("Starting highlighting process");
        log("Input:", { text, language, theme: theme.name });
        setLoading(true);
        const highlighter = yield getHighlighter();
        log("Highlighter instance obtained");
        if (!text || !language || !theme) {
          throw new Error("Missing required props");
        }
        log("Attempting tokenization...");
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme,
          includeExplanation: true
        });
        log("Raw tokenization result:", result);
        if (!(result == null ? void 0 : result.tokens)) {
          throw new Error("Invalid tokenization result");
        }
        const processedTokens = result.tokens.map((line) => {
          if (!Array.isArray(line)) {
            return [line];
          }
          return line.flatMap((token) => splitTokens(token));
        });
        log("Processed tokens:", processedTokens);
        setTokens(processedTokens);
        setError(null);
      } catch (err) {
        const errorMessage = err instanceof Error ? err.message : "Failed to highlight code";
        log("Error during highlighting:", err);
        setError(errorMessage);
        setTokens([]);
      } finally {
        setLoading(false);
      }
    });
    highlight();
  }, [text, language, theme]);
  const renderToken = (token, index) => {
    log("Rendering token:", token);
    return /* @__PURE__ */ React.createElement(
      Text,
      {
        key: `${token.offset || index}-${index}`,
        style: [
          {
            color: token.color,
            fontSize,
            fontFamily: "monospace",
            fontStyle: token.fontStyle === 2 ? "italic" : "normal",
            fontWeight: token.fontStyle === 1 ? "bold" : "normal"
          },
          textStyle
        ]
      },
      token.content
    );
  };
  const renderLine = (lineTokens, index) => {
    log(`Rendering line ${index} with ${lineTokens.length} tokens`);
    const isHighlighted = highlightLines.includes(index + 1);
    return /* @__PURE__ */ React.createElement(
      View,
      {
        key: index,
        style: [
          {
            flexDirection: "row",
            flexWrap: "wrap",
            backgroundColor: isHighlighted ? "rgba(255,255,255,0.1)" : "transparent"
          }
        ]
      },
      lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))
    );
  };
  if (loading) {
    return /* @__PURE__ */ React.createElement(View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(ActivityIndicator, null));
  }
  if (error) {
    return /* @__PURE__ */ React.createElement(View, { style: [{ padding: 10 }, containerStyle] }, /* @__PURE__ */ React.createElement(Text, { style: { color: "#ff6b6b" } }, "Error:", error));
  }
  return /* @__PURE__ */ React.createElement(ScrollView, __spreadProps(__spreadValues({}, scrollViewProps), { horizontal: true, style: [{ backgroundColor: ((_a = theme.colors) == null ? void 0 : _a.background) || "#1e1e1e" }, scrollViewProps == null ? void 0 : scrollViewProps.style], contentContainerStyle: [{ padding: 10 }, containerStyle] }), /* @__PURE__ */ React.createElement(View, { onStartShouldSetResponder: () => true }, tokens.map((lineTokens, index) => renderLine(lineTokens, index))));
};

export { SyntaxHighlighter };
//# sourceMappingURL=SyntaxHighlighter.js.map
