/* eslint-disable no-console */
import type { ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'

import { getHighlighter } from './SyntaxContext'

const DEBUG = true
function log(message: string, ...args: any[]) {
  if (DEBUG) {
    console.log(`[SyntaxHighlighter] ${message}`, ...args)
  }
}

// Helper function to split tokens based on syntax patterns
function splitTokens(token: ThemedToken): ThemedToken[] {
  const patterns = [
    // JSX/TSX patterns
    { regex: /(<\/?\w+)|(\{[^}]*\})|(\/>)|(>)/g, color: '#85e89d' }, // JSX tags and expressions
    { regex: /\b(className|onMouseEnter|onMouseLeave)\b/g, color: '#79b8ff' }, // JSX attributes
    { regex: /['"`].*?['"`]/g, color: '#9ecbff' }, // Strings
    { regex: /\b(return|const|let|var|function)\b/g, color: '#f97583' }, // Keywords
    { regex: /\b(true|false|null|undefined)\b/g, color: '#79b8ff' }, // Constants
    { regex: /[(){}[\]]/g, color: '#e1e4e8' }, // Brackets
  ]

  const result: ThemedToken[] = []
  let lastIndex = 0
  const content = token.content

  // Sort matches by their starting position
  const matches: Array<{ pattern: RegExp, color: string, match: RegExpExecArray }> = []
  patterns.forEach(({ regex, color }) => {
    regex.lastIndex = 0 // Reset regex
    let match
    while ((match = regex.exec(content)) !== null) {
      matches.push({ pattern: regex, color, match })
    }
  })

  // Sort matches by index
  matches.sort((a, b) => a.match.index - b.match.index)

  // Process matches in order
  matches.forEach((matchInfo) => {
    const { match, color } = matchInfo
    const startPos = match.index

    // Add text before the match if there is any
    if (startPos > lastIndex) {
      result.push({
        content: content.slice(lastIndex, startPos),
        color: token.color,
        fontStyle: token.fontStyle,
        offset: token.offset + lastIndex,
      })
    }

    // Add the matched text with its special color
    result.push({
      content: match[0],
      color,
      fontStyle: token.fontStyle,
      offset: token.offset + startPos,
    })

    lastIndex = startPos + match[0].length
  })

  // Add any remaining text
  if (lastIndex < content.length) {
    result.push({
      content: content.slice(lastIndex),
      color: token.color,
      fontStyle: token.fontStyle,
      offset: token.offset + lastIndex,
    })
  }

  return result.filter(t => t.content.length > 0)
}

export interface SyntaxHighlighterProps {
  text: string
  language: string
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
  highlightLines?: number[]
}

const defaultHighlightLines: number[] = []

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({
  text,
  language,
  theme,
  fontSize = 14,
  textStyle,
  scrollViewProps,
  containerStyle,
  highlightLines = defaultHighlightLines,
}) => {
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])

  React.useEffect(() => {
    const highlight = async () => {
      try {
        log('Starting highlighting process')
        log('Input:', { text, language, theme: theme.name })

        setLoading(true)
        const highlighter = await getHighlighter()
        log('Highlighter instance obtained')

        if (!text || !language || !theme) {
          throw new Error('Missing required props')
        }

        log('Attempting tokenization...')
        const result = highlighter.codeToTokens(text, {
          lang: language,
          theme,
          includeExplanation: true,
        })

        log('Raw tokenization result:', result)

        if (!result?.tokens) {
          throw new Error('Invalid tokenization result')
        }

        // Process each line of tokens
        const processedTokens = result.tokens.map((line) => {
          if (!Array.isArray(line)) {
            return [line]
          }
          // Split each token in the line
          return line.flatMap(token => splitTokens(token))
        })

        log('Processed tokens:', processedTokens)

        setTokens(processedTokens)
        setError(null)
      }
      catch (err) {
        const errorMessage = err instanceof Error ? err.message : 'Failed to highlight code'
        log('Error during highlighting:', err)
        setError(errorMessage)
        setTokens([])
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    log('Rendering token:', token)
    return (
      <Text
        key={`${token.offset || index}-${index}`}
        style={[
          {
            color: token.color,
            fontSize,
            fontFamily: 'monospace',
            fontStyle: token.fontStyle === 2 ? 'italic' : 'normal',
            fontWeight: token.fontStyle === 1 ? 'bold' : 'normal',
          },
          textStyle,
        ]}
      >
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => {
    log(`Rendering line ${index} with ${lineTokens.length} tokens`)
    const isHighlighted = highlightLines.includes(index + 1)

    return (
      <View
        key={index}
        style={[
          {
            flexDirection: 'row',
            flexWrap: 'wrap',
            backgroundColor: isHighlighted ? 'rgba(255,255,255,0.1)' : 'transparent',
          },
        ]}
      >
        {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
      </View>
    )
  }

  if (loading) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor: theme.colors?.background || '#1e1e1e' }, scrollViewProps?.style]} contentContainerStyle={[{ padding: 10 }, containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}
